#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from pyramid.threadlocal import get_current_registry
from pyramid_es.mixin import ESField, ESMapping, ElasticMixin as ElasticMixinBase
from zope.interface import classImplements
from zope.intid.interfaces import IIntIds

from pyams_content.shared.common import WfSharedContent
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_content.shared.common.interfaces.types import IWfTypedSharedContent
from pyams_content_es.interfaces import IDocumentIndexInfo, IDocumentIndexTarget
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import get_utility
from pyams_workflow.interfaces import IWorkflowState


class ElasticMixin(ElasticMixinBase):
    """ElasticSearch base mixin class"""

    @property
    def id(self):
        oid = ISequentialIdInfo(self).hex_oid
        state = IWorkflowState(self, None)
        if state is None:
            return oid
        else:
            return '{oid}.{version}'.format(oid=oid, version=state.version_id)

    @property
    def internal_id(self):
        intids = get_utility(IIntIds)
        return intids.register(self)

    @property
    def reference_id(self):
        return ISequentialIdInfo(self).hex_oid

    def elastic_mapping(self):
        return IDocumentIndexInfo(self)

    def elastic_document(self):
        document_info = super(ElasticMixin, self).elastic_document()
        registry = get_current_registry()
        for name, adapted in registry.getAdapters((self,), IDocumentIndexInfo):
            if not name:
                continue
            document_info.update(adapted)
        return document_info


WfSharedContent.__bases__ += (ElasticMixin,)
classImplements(WfSharedContent, IDocumentIndexTarget)


@adapter_config(context=IWfSharedContent, provides=IDocumentIndexInfo)
def wf_shared_content_index_info(content):
    return ESMapping(analyzer='content',
                     properties=ESMapping(ESField('internal_id'),
                                          ESField('reference_id'),
                                          ESField('content_type'),
                                          ESField('title', boost=3.0),
                                          ESField('short_name'),
                                          ESField('description'),
                                          ESField('keywords', boost=2.0)))


@adapter_config(context=IWfTypedSharedContent, provides=IDocumentIndexInfo)
def wf_typed_shared_content_index_info(content):
    return ESMapping(analyzer='content',
                     properties=ESMapping(ESField('internal_id'),
                                          ESField('reference_id'),
                                          ESField('content_type'),
                                          ESField('data_type'),
                                          ESField('title', boost=3.0),
                                          ESField('short_name'),
                                          ESField('description'),
                                          ESField('keywords', boost=2.0)))
